/**   @file  suite_clusterlst.c
 *   @brief  Check testsuite for clusterlst functions
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-11-12 19:48
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include <stdlib.h>
#include "checktools.inc"

#include "clusterlst.c"

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(clusterlst_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

START_TEST(clusterlst_uninitialized_init) {
    clusterrecord_t* first=clusterlistfirst;
    clusterrecord_t* last=clusterlistlast;
    clusterrecord_t* cursor=clusterlistcursor;
    void       (*f_add)(cluster_t* cluster)=clusterlist_add;
    cluster_t* (*f_find) (const char* host)=clusterlist_find;
    cluster_t* (*f_first)()=clusterlist_first;
    cluster_t* (*f_next)()=clusterlist_next;
    cluster_t* (*f_get)()=clusterlist_get;

    clusterlist_init();

    ck_assert(first!=clusterlistfirst);
    ck_assert(last!=clusterlistlast);
    ck_assert(cursor!=clusterlistcursor);
    ck_assert(f_add!=clusterlist_add);
    ck_assert(f_find!=clusterlist_find);
    ck_assert(f_first!=clusterlist_first);
    ck_assert(f_next!=clusterlist_next);
    ck_assert(f_get!=clusterlist_get);
}
END_TEST

START_TEST(clusterlst_uninitialized_add) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    clusterrecord_t* first=clusterlistfirst;
    clusterrecord_t* last=clusterlistlast;
    clusterrecord_t* cursor=clusterlistcursor;
    void       (*f_add)(cluster_t* cluster)=clusterlist_add;
    cluster_t* (*f_find) (const char* host)=clusterlist_find;
    cluster_t* (*f_first)()=clusterlist_first;
    cluster_t* (*f_next)()=clusterlist_next;
    cluster_t* (*f_get)()=clusterlist_get;
    cluster_t* cluster;

    if (NULL==(cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    cluster->host=NULL;
    cluster->user=NULL;
    cluster->pass=NULL;
    cluster->insecure=NULL;
    cluster->cacert=NULL;
    clusterlist_add(cluster);

    ck_assert(first!=clusterlistfirst);
    ck_assert(last!=clusterlistlast);
    ck_assert(cursor!=clusterlistcursor);
    ck_assert(f_add!=clusterlist_add);
    ck_assert(f_find!=clusterlist_find);
    ck_assert(f_first!=clusterlist_first);
    ck_assert(f_next!=clusterlist_next);
    ck_assert(f_get!=clusterlist_get);

    /* Second run should not invoke init and change the pointers and functors */
    first=clusterlistfirst;
    last=clusterlistlast;
    cursor=clusterlistcursor;
    f_add=clusterlist_add;
    f_find=clusterlist_find;
    f_first=clusterlist_first;
    f_next=clusterlist_next;
    f_get=clusterlist_get;

    if (NULL==(cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    cluster->host=NULL;
    cluster->user=NULL;
    cluster->pass=NULL;
    cluster->insecure=NULL;
    cluster->cacert=NULL;
    clusterlist_add(cluster);

    ck_assert(first==clusterlistfirst);
    /* Change expected ck_assert(last==clusterlistlast);*/
    ck_assert(cursor==clusterlistcursor);
    ck_assert(f_add==clusterlist_add);
    ck_assert(f_find==clusterlist_find);
    ck_assert(f_first==clusterlist_first);
    ck_assert(f_next==clusterlist_next);
    ck_assert(f_get==clusterlist_get);
}
END_TEST

START_TEST(clusterlst_uninitialized_find) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    clusterrecord_t* first=clusterlistfirst;
    clusterrecord_t* last=clusterlistlast;
    clusterrecord_t* cursor=clusterlistcursor;
    void       (*f_add)(cluster_t* cluster)=clusterlist_add;
    cluster_t* (*f_find) (const char* host)=clusterlist_find;
    cluster_t* (*f_first)()=clusterlist_first;
    cluster_t* (*f_next)()=clusterlist_next;
    cluster_t* (*f_get)()=clusterlist_get;
    cluster_t* cluster;

    cluster = clusterlist_find("");

    ck_assert(first!=clusterlistfirst);
    ck_assert(last!=clusterlistlast);
    ck_assert(cursor!=clusterlistcursor);
    ck_assert(f_add!=clusterlist_add);
    ck_assert(f_find!=clusterlist_find);
    ck_assert(f_first!=clusterlist_first);
    ck_assert(f_next!=clusterlist_next);
    ck_assert(f_get!=clusterlist_get);
    ck_assert(NULL==cluster);

    /* Second run should not invoke init and change the pointers and functors */
    first=clusterlistfirst;
    last=clusterlistlast;
    cursor=clusterlistcursor;
    f_add=clusterlist_add;
    f_find=clusterlist_find;
    f_first=clusterlist_first;
    f_next=clusterlist_next;
    f_get=clusterlist_get;

    cluster = clusterlist_find("");

    ck_assert(first==clusterlistfirst);
    /* Change expected ck_assert(last==clusterlistlast);*/
    ck_assert(cursor==clusterlistcursor);
    ck_assert(f_add==clusterlist_add);
    ck_assert(f_find==clusterlist_find);
    ck_assert(f_first==clusterlist_first);
    ck_assert(f_next==clusterlist_next);
    ck_assert(f_get==clusterlist_get);
    f_get=clusterlist_get;
}
END_TEST

START_TEST(clusterlst_uninitialized_first) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    clusterrecord_t* first=clusterlistfirst;
    clusterrecord_t* last=clusterlistlast;
    clusterrecord_t* cursor=clusterlistcursor;
    void       (*f_add)(cluster_t* cluster)=clusterlist_add;
    cluster_t* (*f_find) (const char* host)=clusterlist_find;
    cluster_t* (*f_first)()=clusterlist_first;
    cluster_t* (*f_next)()=clusterlist_next;
    cluster_t* (*f_get)()=clusterlist_get;
    cluster_t* cluster;

    cluster = clusterlist_first();

    ck_assert(first!=clusterlistfirst);
    ck_assert(last!=clusterlistlast);
    ck_assert(cursor!=clusterlistcursor);
    ck_assert(f_add!=clusterlist_add);
    ck_assert(f_find!=clusterlist_find);
    ck_assert(f_first!=clusterlist_first);
    ck_assert(f_next!=clusterlist_next);
    ck_assert(f_get!=clusterlist_get);
    ck_assert(NULL==cluster);

    /* Second run should not invoke init and change the pointers and functors */
    first=clusterlistfirst;
    last=clusterlistlast;
    cursor=clusterlistcursor;
    f_add=clusterlist_add;
    f_find=clusterlist_find;
    f_first=clusterlist_first;
    f_next=clusterlist_next;
    f_get=clusterlist_get;

    cluster = clusterlist_first();

    ck_assert(first==clusterlistfirst);
    /* Change expected ck_assert(last==clusterlistlast);*/
    ck_assert(cursor==clusterlistcursor);
    ck_assert(f_add==clusterlist_add);
    ck_assert(f_find==clusterlist_find);
    ck_assert(f_first==clusterlist_first);
    ck_assert(f_next==clusterlist_next);
    ck_assert(f_get==clusterlist_get);
    f_get=clusterlist_get;
}
END_TEST

START_TEST(clusterlst_uninitialized_next) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    clusterrecord_t* first=clusterlistfirst;
    clusterrecord_t* last=clusterlistlast;
    clusterrecord_t* cursor=clusterlistcursor;
    void       (*f_add)(cluster_t* cluster)=clusterlist_add;
    cluster_t* (*f_find) (const char* host)=clusterlist_find;
    cluster_t* (*f_first)()=clusterlist_first;
    cluster_t* (*f_next)()=clusterlist_next;
    cluster_t* (*f_get)()=clusterlist_get;
    cluster_t* cluster;

    cluster = clusterlist_next();

    ck_assert(first!=clusterlistfirst);
    ck_assert(last!=clusterlistlast);
    ck_assert(cursor!=clusterlistcursor);
    ck_assert(f_add!=clusterlist_add);
    ck_assert(f_find!=clusterlist_find);
    ck_assert(f_first!=clusterlist_first);
    ck_assert(f_next!=clusterlist_next);
    ck_assert(f_get!=clusterlist_get);
    ck_assert(NULL==cluster);

    /* Second run should not invoke init and change the pointers and functors */
    first=clusterlistfirst;
    last=clusterlistlast;
    cursor=clusterlistcursor;
    f_add=clusterlist_add;
    f_find=clusterlist_find;
    f_first=clusterlist_first;
    f_next=clusterlist_next;
    f_get=clusterlist_get;

    cluster = clusterlist_next();

    ck_assert(first==clusterlistfirst);
    /* Change expected ck_assert(last==clusterlistlast);*/
    ck_assert(cursor==clusterlistcursor);
    ck_assert(f_add==clusterlist_add);
    ck_assert(f_find==clusterlist_find);
    ck_assert(f_first==clusterlist_first);
    ck_assert(f_next==clusterlist_next);
    ck_assert(f_get==clusterlist_get);
    f_get=clusterlist_get;
}
END_TEST

START_TEST(clusterlst_uninitialized_get) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    clusterrecord_t* first=clusterlistfirst;
    clusterrecord_t* last=clusterlistlast;
    clusterrecord_t* cursor=clusterlistcursor;
    void       (*f_add)(cluster_t* cluster)=clusterlist_add;
    cluster_t* (*f_find) (const char* host)=clusterlist_find;
    cluster_t* (*f_first)()=clusterlist_first;
    cluster_t* (*f_next)()=clusterlist_next;
    cluster_t* (*f_get)()=clusterlist_get;
    cluster_t* cluster;

    cluster = clusterlist_get();

    ck_assert(first!=clusterlistfirst);
    ck_assert(last!=clusterlistlast);
    ck_assert(cursor!=clusterlistcursor);
    ck_assert(f_add!=clusterlist_add);
    ck_assert(f_find!=clusterlist_find);
    ck_assert(f_first!=clusterlist_first);
    ck_assert(f_next!=clusterlist_next);
    ck_assert(f_get!=clusterlist_get);
    ck_assert(NULL==cluster);

    /* Second run should not invoke init and change the pointers and functors */
    first=clusterlistfirst;
    last=clusterlistlast;
    cursor=clusterlistcursor;
    f_add=clusterlist_add;
    f_find=clusterlist_find;
    f_first=clusterlist_first;
    f_next=clusterlist_next;
    f_get=clusterlist_get;

    cluster = clusterlist_get();

    ck_assert(first==clusterlistfirst);
    /* Change expected ck_assert(last==clusterlistlast);*/
    ck_assert(cursor==clusterlistcursor);
    ck_assert(f_add==clusterlist_add);
    ck_assert(f_find==clusterlist_find);
    ck_assert(f_first==clusterlist_first);
    ck_assert(f_next==clusterlist_next);
    ck_assert(f_get==clusterlist_get);
    f_get=clusterlist_get;
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void clusterlst_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void clusterlst_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
void clusterlst_unchecked_common_setup() {
    forktest_init();
}
void clusterlst_unchecked_common_teardown() {
}
/************************************************************************/
Suite* clusterlst_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("Clusterlst");

    tc = tcase_create("ClusterlstUninitialized");
    tcase_set_tags(tc,"ClusterlstTag ClusterlstTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, clusterlst_checked_uninitialized_setup, clusterlst_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, clusterlst_unchecked_common_setup, clusterlst_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, clusterlst_test,6);
    tcase_add_test(tc, clusterlst_uninitialized_init);
    tcase_add_test(tc, clusterlst_uninitialized_add);
    tcase_add_test(tc, clusterlst_uninitialized_find);
    tcase_add_test(tc, clusterlst_uninitialized_first);
    tcase_add_test(tc, clusterlst_uninitialized_next);
    tcase_add_test(tc, clusterlst_uninitialized_get);

    return s;
}
/* vim: set tw=80: */
